#!/usr/bin/env cwl-runner
cwlVersion: v1.2
class: Workflow
requirements:
  StepInputExpressionRequirement: {}
  InlineJavascriptRequirement: {}
  MultipleInputFeatureRequirement: {}
  SubworkflowFeatureRequirement: {}
  ScatterFeatureRequirement: {}

label: (Hybrid) Metagenomics workflow
doc: |
  **Workflow (hybrid) metagenomic assembly and binning  **<br>
    - Workflow Illumina Quality: https://workflowhub.eu/workflows/336?version=1	
      - FastQC (control)
      - fastp (quality trimming)
      - kraken2 (taxonomy)
      - bbmap contamination filter
    - Workflow Longread Quality:	
      - NanoPlot (control)
      - filtlong (quality trimming)
      - kraken2 (taxonomy)
      - minimap2 contamination filter
    - Kraken2 taxonomic classification of FASTQ reads
    - SPAdes/Flye (Assembly)
    - QUAST (Assembly quality report)

    (optional)
    - Workflow binnning https://workflowhub.eu/workflows/64?version=11
      - Metabat2/MaxBin2/SemiBin
      - DAS Tool
      - CheckM
      - BUSCO
      - GTDB-Tk

    (optional)
    - Workflow Genome-scale metabolic models https://workflowhub.eu/workflows/372
      - CarveMe (GEM generation)
      - MEMOTE (GEM test suite)
      - SMETANA (Species METabolic interaction ANAlysis)

  Other UNLOCK workflows on WorkflowHub: https://workflowhub.eu/projects/16/workflows?view=default<br><br>

  **All tool CWL files and other workflows can be found here:**<br>
    Tools: https://gitlab.com/m-unlock/cwl/-/tree/master/cwl <br>
    Workflows: https://gitlab.com/m-unlock/cwl/-/tree/master/cwl/workflows<br>

  **How to setup and use an UNLOCK workflow:**<br>
  https://m-unlock.gitlab.io/docs/setup/setup.html<br>

outputs:
  read_filtering_output_keep:
    label: Read filtering output
    doc: Read filtering stats + filtered reads
    type: Directory?
    outputSource: keep_readfilter_files_to_folder/results
  read_filtering_output:
    label: Read filtering output
    doc: Read filtering stats + filtered reads
    type: Directory?
    outputSource: readfilter_files_to_folder/results
  assembly_output:
    label: Assembly output
    doc: Output from different assembly steps
    type: Directory
    outputSource: assembly_files_to_folder/results
  # bam_output:
  #   label: BAM files
  #   doc: Mapping results in indexed BAM format
  #   type: Directory?
  #   outputSource: sorted_bam_files_to_folder/results
  binning_output:
    label: Binning output
    doc: Binning outputfolders
    type: Directory?
    outputSource: binning_files_to_folder/results
  gem_output:
    label: Community GEM output
    doc: Community GEM output folder
    type: Directory?
    outputSource: GEM_files_to_folder/results

inputs:
  identifier:
    type: string
    label: Identifier
    doc: Identifier for this dataset used in this workflow (required)
  illumina_forward_reads:
    type: File[]?
    doc: Illumina Forward sequence file(s)
    label: Forward reads
    loadListing: no_listing
  illumina_reverse_reads:
    type: File[]?
    label: Reverse reads
    doc: Illumina Reverse sequence file(s)
    loadListing: no_listing
  pacbio_reads:
    type: File[]?
    label: PacBio reads
    doc: File(s) with PacBio reads in FASTQ format
    loadListing: no_listing
  nanopore_reads:
    type: File[]?
    label: Oxford Nanopore reads
    doc: File(s) with Oxford Nanopore reads in FASTQ format
    loadListing: no_listing
  fastq_rich:
    type: boolean
    doc: | 
          Input fastq is generated by albacore, MinKNOW or guppy  with additional information concerning channel and time. 
          Used to creating more informative quality plots (default false)
    label: Fastq rich (ONT)
    default: false

# Longread defaults taken from nf-core MAG
  longread_minimum_length:
    type: int?
    doc: Minimum read length threshold (default 1000)
    label: Minimum read length
    default: 1000
  longread_keep_percent:
    type: float?
    doc: Keep only this percentage of the best reads (measured by bases) (default 90)
    label: Keep percentage
    default: 90
  longread_length_weight:
    type: float?
    doc: Weight given to the length score (default 10)
    label: Length weigth
    default: 10

  filter_references:
    type: File[]?
    doc: Reference fasta file(s) used for pre-filtering. Can be gzipped (not mixed)
    label: Reference file(s)
    loadListing: no_listing
  use_reference_mapped_reads:
    type: boolean?
    doc: Continue with reads mapped to the given reference (default false)
    label: Keep mapped reads
    default: false
  keep_filtered_reads:
    type: boolean?
    doc: Keep filtered reads in the final output (default false)
    label: Keep filtered reads
    default: false
  deduplicate:
    type: boolean?
    doc: Remove exact duplicate reads Illumina reads with fastp (default false)
    label: Deduplicate reads
    default: false
  kraken2_confidence:
    type: float?
    label: Kraken2 confidence threshold
    doc: Confidence score threshold must be in [0, 1] (default 0.0) 
  kraken2_database:
    type: Directory[]?
    doc: Database location of kraken2
    label: Kraken2 database
    default: []
    loadListing: no_listing
  gtdbtk_data:
    type: Directory?
    doc: Directory containing the GTDBTK repository
    label: gtdbtk data directory
    loadListing: no_listing
  busco_data:
    type: Directory?
    label: BUSCO dataset
    doc: Path to the BUSCO dataset downloaded location
    loadListing: no_listing
  ont_basecall_model:
    type: string?
    label: ONT Basecalling model for MEDAKA
    doc: |
      Used in MEDAKA
      Basecalling model used with guppy default r941_min_high.
      Available: r103_fast_g507, r103_fast_snp_g507, r103_fast_variant_g507, r103_hac_g507, r103_hac_snp_g507, r103_hac_variant_g507, r103_min_high_g345, r103_min_high_g360, r103_prom_high_g360, r103_prom_snp_g3210, r103_prom_variant_g3210, r103_sup_g507, r103_sup_snp_g507, r103_sup_variant_g507, r1041_e82_400bps_fast_g615, r1041_e82_400bps_fast_variant_g615, r1041_e82_400bps_hac_g615, r1041_e82_400bps_hac_variant_g615, r1041_e82_400bps_sup_g615, r1041_e82_400bps_sup_variant_g615, r104_e81_fast_g5015, r104_e81_fast_variant_g5015, r104_e81_hac_g5015, r104_e81_hac_variant_g5015, r104_e81_sup_g5015, r104_e81_sup_g610, r104_e81_sup_variant_g610, r10_min_high_g303, r10_min_high_g340, r941_e81_fast_g514, r941_e81_fast_variant_g514, r941_e81_hac_g514, r941_e81_hac_variant_g514, r941_e81_sup_g514, r941_e81_sup_variant_g514, r941_min_fast_g303, r941_min_fast_g507, r941_min_fast_snp_g507, r941_min_fast_variant_g507, r941_min_hac_g507, r941_min_hac_snp_g507, r941_min_hac_variant_g507, r941_min_high_g303, r941_min_high_g330, r941_min_high_g340_rle, r941_min_high_g344, r941_min_high_g351, r941_min_high_g360, r941_min_sup_g507, r941_min_sup_snp_g507, r941_min_sup_variant_g507, r941_prom_fast_g303, r941_prom_fast_g507, r941_prom_fast_snp_g507, r941_prom_fast_variant_g507, r941_prom_hac_g507, r941_prom_hac_snp_g507, r941_prom_hac_variant_g507, r941_prom_high_g303, r941_prom_high_g330, r941_prom_high_g344, r941_prom_high_g360, r941_prom_high_g4011, r941_prom_snp_g303, r941_prom_snp_g322, r941_prom_snp_g360, r941_prom_sup_g507, r941_prom_sup_snp_g507, r941_prom_sup_variant_g507, r941_prom_variant_g303, r941_prom_variant_g322, r941_prom_variant_g360, r941_sup_plant_g610, r941_sup_plant_variant_g610
      (required for Medaka)
  pilon_fixlist:
    type: string?
    label: Pilon fix list
    doc: |
      A comma-separated list of categories of issues to try to fix:
        "snps": try to fix individual base errors;
        "indels": try to fix small indels;
        "gaps": try to fill gaps;
        "local": try to detect and fix local misassemblies;
        "all": all of the above (default);
        "bases": shorthand for "snps" and "indels" (for back compatibility);
        default; snps,gaps,local (conservative)
    default: "snps,gaps,local"
  genome_size:
    type: string?
    label: Genome Size
    doc: Estimated genome size (for example, 5m or 2.6g)
  metagenome:
    type: boolean?
    default: true
    doc: Metagenome option for assemblers (default true)
    label: When working with metagenomes
  semibin_environment:
    type: string?
    doc: SemiBin built-in models; human_gut/dog_gut/ocean/soil/cat_gut/human_oral/mouse_gut/pig_gut/built_environment/wastewater/chicken_caecum/global (default global)
    label: SemiBin Environment
    default: global
  # TODO Selection for binning to use spades or flye
  run_spades:
    type: boolean?
    label: Use SPAdes
    doc: Run with SPAdes assembler (default true)
    default: true
  only_assembler_mode_spades:
    type: boolean?
    label: Only spades assembler
    doc: Run spades in only assembler mode (without read error correction) (default false)
    default: false
  run_flye:
    type: boolean?
    label: Use Flye
    doc: Run with Flye assembler (default false)
    default: false
  run_pilon:
    type: boolean?
    label: Use Pilon
    doc: Run with Pilon illumina assembly polishing (default false)
    default: false
  run_medaka:
    type: boolean?
    label: Use Medaka
    doc: Run with Mekada assembly polishing with nanopore reads (default false)
    default: false
  binning:
    type: boolean?
    label: Run binning workflow
    doc: Run with contig binning workflow (default false)
    default: false
  run_GEM:
    type: boolean?
    label: Run GEM workflow
    doc: | 
          Run the community GEnomescale Metabolic models workflow on bins. (default false)
          NOTE: Uses by default private docker containers 
    default: false
  run_smetana:
    type: boolean?
    label: Run SMETANA
    doc: Run SMETANA (Species METabolic interaction ANAlysis) (default false)
    default: false
  skip_qc_unfiltered:
    type: boolean?
    doc: Skip quality analyses of unfiltered input reads (default false)
    label: Skip QC unfiltered
    default: false

  threads:
    type: int?
    doc: Number of threads to use for computational processes (default 2)
    label: Number of threads
    default: 2
  memory:
    type: int?
    doc: Maximum memory usage in megabytes (default 8GB)
    label: Memory usage (MB)
    default: 8000

  destination:
    type: string?
    label: Output Destination (prov only)
    doc: Not used in this workflow. Output destination used for cwl-prov reporting only.

steps:
#############################################
#### Preparation of reference files
  prepare_fasta_db:
    label: Prepare references
    doc: Prepare references to a single fasta file and unique headers
    when: $(inputs.fasta_files !== null && inputs.fasta_files.length !== 0)
    run: ../tools/bbmap/prepare_fasta_db.cwl
    in:
      fasta_files: filter_references
      output_file_name:
        valueFrom: "filter-reference_prepared.fa.gz"
    out: [fasta_db]

#############################################
#### Quality workflow ILLUMINA
  workflow_quality_illumina:
    label: Quality and filtering workflow
    doc: Quality, filtering and taxonomic classification of Illumina reads
    when: $(inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: workflow_illumina_quality.cwl
    in:
      identifier: identifier
      forward_reads: illumina_forward_reads
      reverse_reads: illumina_reverse_reads
      skip_qc_unfiltered: skip_qc_unfiltered
      deduplicate: deduplicate
      prepare_reference:
        default: false
      filter_references:
        linkMerge: merge_flattened
        source: [prepare_fasta_db/fasta_db]
        pickValue: all_non_null
      keep_reference_mapped_reads: use_reference_mapped_reads
      kraken2_database: kraken2_database
      kraken2_confidence: kraken2_confidence
      memory: memory
      threads: threads
      step:
        default: 1
    out: [QC_reverse_reads, QC_forward_reads, reports_folder, kraken2_folder]

#############################################
#### Quality workflow Oxford Nanopore
  workflow_quality_nanopore:
    label: Oxford Nanopore quality workflow
    doc: Quality, filtering and taxonomic classification workflow for Oxford Nanopore reads
    when: $(inputs.longreads !== null && inputs.longreads !== 0)
    run: workflow_longread_quality.cwl
    in:
      identifier: identifier
      longreads: nanopore_reads
      fastq_rich: fastq_rich
      readtype:
        default: "Nanopore"
      skip_qc_unfiltered: skip_qc_unfiltered
      keep_percent: longread_keep_percent
      minimum_length: longread_minimum_length
      length_weight: longread_length_weight
      prepare_reference:
        default: false
      filter_references:
        linkMerge: merge_flattened
        source: [prepare_fasta_db/fasta_db]
        pickValue: all_non_null
      keep_reference_mapped_reads: use_reference_mapped_reads
      kraken2_database: kraken2_database
      kraken2_confidence: kraken2_confidence
      threads: threads
      step:
        default: 1
    out: [filtered_reads, filtlong_log, reference_filter_longreads_log, nanoplot_unfiltered_folder, nanoplot_filtered_folder, kraken2_folder]
#############################################
#### Quality workflow PacBio
  workflow_quality_pacbio:
    label: PacBio quality and filtering workflow
    doc: Quality, filtering and taxonomic classification for PacBio reads
    when: $(inputs.longreads !== null && inputs.longreads !== 0)
    run: workflow_longread_quality.cwl
    in:
      identifier: identifier
      longreads: pacbio_reads
      readtype:
        default: "PacBio"
      skip_qc_unfiltered: skip_qc_unfiltered
      keep_percent: longread_keep_percent
      minimum_length: longread_minimum_length
      length_weight: longread_length_weight
      prepare_reference:
        default: false
      filter_references:
        linkMerge: merge_flattened
        source: [prepare_fasta_db/fasta_db]
      keep_reference_mapped_reads: use_reference_mapped_reads
      kraken2_database: kraken2_database
      kraken2_confidence: kraken2_confidence
      threads: threads
      step:
        default: 1
    out: [filtered_reads, filtlong_log, reference_filter_longreads_log, nanoplot_unfiltered_folder, nanoplot_filtered_folder, kraken2_folder]

#############################################
#### Assembly using SPAdes
  spades:
    doc: Genome assembly using SPAdes with illumina and or long reads
    label: SPAdes assembly
    when: $(inputs.run_spades && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: ../tools/spades/spades.cwl
    in:
      run_spades: run_spades

      forward_reads:
        source: [ workflow_quality_illumina/QC_forward_reads ]
        linkMerge: merge_nested
      reverse_reads:
        source: [ workflow_quality_illumina/QC_reverse_reads ]
        linkMerge: merge_nested
      pacbio_reads:
        source: workflow_quality_pacbio/filtered_reads
        valueFrom:
          ${
            var reads = null;
            if (self !== null) { reads = [self]; }
            return reads;
          }
      nanopore_reads:
        source: workflow_quality_nanopore/filtered_reads
        valueFrom:
          ${
            var reads = null;
            if (self !== null) { reads = [self]; }
            return reads;
          }
      only_assembler: only_assembler_mode_spades
      metagenome: metagenome
      memory: memory
      threads: threads
    out: [contigs, scaffolds, assembly_graph, contigs_assembly_paths, scaffolds_assembly_paths, contigs_before_rr, params, log, internal_config, internal_dataset]

  compress_spades:
    label: SPAdes compressed
    doc: Compress the large Spades assembly output files
    when: $(inputs.run_spades && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: ../tools/bash/pigz.cwl
    scatter: [inputfile]
    scatterMethod: dotproduct
    in:
      run_spades: run_spades
      forward_reads: illumina_forward_reads

      threads: threads
      inputfile:
        source: [spades/contigs, spades/scaffolds,spades/assembly_graph,spades/contigs_before_rr, spades/contigs_assembly_paths,spades/scaffolds_assembly_paths]
        linkMerge: merge_flattened
        pickValue: all_non_null
    out: [outfile]

#############################################
#### De novo assembly with Flye
  flye:
    label: Flye assembly
    doc: De novo assembly of single-molecule reads with Flye
    when: $(inputs.run_flye)
    run: ../tools/flye/flye.cwl
    in:
      run_flye: run_flye

      nano_raw: workflow_quality_nanopore/filtered_reads
      pacbio_raw: workflow_quality_pacbio/filtered_reads
      threads: threads
      metagenome: metagenome
      genome_size: genome_size
    out: [00_assembly, 10_consensus, 20_repeat, 30_contigger, 40_polishing, assembly, assembly_info, flye_log, params]

#############################################
#### Polishing of assembled genome with Medaka
  medaka:
    label: Medaka polishing of assembly
    doc: Medaka for (ont reads) polishing of a assembled genome
    when: $(inputs.run_medaka && inputs.nanopore_reads !== null && inputs.nanopore_reads.length !== 0)
    run: ../tools/medaka/medaka_consensus_py.cwl
    in:
      run_medaka: run_medaka

      nanopore_reads: nanopore_reads
      threads: threads
      draft_assembly:
        source:
        - flye/assembly
        - spades/scaffolds
        pickValue: first_non_null
      reads: workflow_quality_nanopore/filtered_reads
      basecall_model: ont_basecall_model
    out: [polished_assembly, gaps_in_draft_coords] # probs, calls_to_draft
  metaquast_medaka:
    label: assembly evaluation
    doc: evaluation of polished assembly with metaQUAST
    when: $(inputs.run_medaka && inputs.nanopore_reads !== null && inputs.nanopore_reads !== 0)
    run: ../tools/quast/metaquast.cwl
    in:
      run_medaka: run_medaka
      nanopore_reads: nanopore_reads
      assembly: medaka/polished_assembly
      threads: threads
    out: [metaquast_outdir, meta_combined_ref, meta_icarusDir, metaquast_krona, not_aligned, meta_downloaded_ref, runs_per_reference, meta_summary, meta_icarus, metaquast_log, metaquast_report, basicStats, quast_icarusDir, quast_icarusHtml, quastReport, quastLog, transposedReport]

#############################################
#### Workflow Pilon assembly polishing
  workflow_pilon:
    label: Pilon worklow
    doc: Illumina reads assembly polishing with Pilon
    when: $(inputs.run_pilon && inputs.illumina_forward_reads !== null && inputs.illumina_forward_reads.length !== 0)
    run: workflow_pilon_mapping.cwl
    in:
      run_pilon: run_pilon

      identifier:
        source: identifier
        valueFrom: $(self)_scaffolds
      assembly:
        source:
        - medaka/polished_assembly
        - flye/assembly
        - spades/scaffolds
        pickValue: first_non_null
      illumina_forward_reads: workflow_quality_illumina/QC_forward_reads
      illumina_reverse_reads: workflow_quality_illumina/QC_reverse_reads
      fixlist: pilon_fixlist
      threads: threads
      memory: memory
    out: [pilon_polished_assembly, vcf, log]

#############################################
# #### Assembly quality assessment using quast
#   quast:
#     doc: Genome assembly quality assessment using Quast
#     label: Quast workflow
#     run: ../tools/quast/quast.cwl
#     in:
#       assembly: workflow_pilon/pilon_polished_assembly
#     out: [basicStats, icarusDir, icarusHtml, quastReport, quastLog, transposedReport]
#############################################
#### Assembly evaluation with QUAST
  metaquast_pilon:
    label: Illumina assembly evaluation
    doc: Illumina evaluation of pilon polished assembly with metaQUAST
    when: $(inputs.run_pilon && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: ../tools/quast/metaquast.cwl
    in:
      run_pilon: run_pilon
      forward_reads: illumina_forward_reads

      assembly: workflow_pilon/pilon_polished_assembly
      threads: threads
    out: [metaquast_outdir, meta_combined_ref, meta_icarusDir, metaquast_krona, not_aligned, meta_downloaded_ref, runs_per_reference, meta_summary, meta_icarus, metaquast_log, metaquast_report, basicStats, quast_icarusDir, quast_icarusHtml, quastReport, quastLog, transposedReport]

#############################################
#### Read mapping using BBmap for binning
  bbmap:
    label: BBmap read mapping
    doc: Illumina read mapping using BBmap on assembled contigs
    when: $(inputs.binning && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: ../tools/bbmap/bbmap.cwl
    in:
      binning: binning

      identifier: identifier
      reference:
        source:
        - workflow_pilon/pilon_polished_assembly
        - medaka/polished_assembly
        - flye/assembly
        - spades/scaffolds
        pickValue: first_non_null
      forward_reads: workflow_quality_illumina/QC_forward_reads
      reverse_reads: workflow_quality_illumina/QC_reverse_reads
      memory: memory
      threads: threads
    out: [sam, stats, covstats, log]
#############################################
#### Convert sam file to sorted bam for binning
  sam_to_sorted_bam:
    label: sam conversion to sorted bam
    doc: Sam file conversion to a sorted indexed bam file
    when: $(inputs.binning && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: ../tools/samtools/sam_to_sorted-bam.cwl
    in:
      binning: binning
      forward_reads: illumina_forward_reads

      identifier: identifier
      sam: bbmap/sam
      threads: threads
    out: [sortedbam]
#############################################
#### reports per contig alignment statistics
  contig_read_counts:
    label: Samtools idxstats
    doc: Reports alignment summary statistics
    when: $(inputs.binning && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: ../tools/samtools/samtools_idxstats.cwl
    in:
      binning: binning
      forward_reads: illumina_forward_reads

      identifier: identifier
      bam_file: sam_to_sorted_bam/sortedbam
      threads: threads
    out: [contigReadCounts]

#############################################
#### Binning workflow
  workflow_binning:
    label: Binning workflow
    doc: Binning workflow to create bins
    when: $(inputs.binning && inputs.forward_reads !== null && inputs.forward_reads.length !== 0)
    run: workflow_metagenomics_binning.cwl
    in:
      binning: binning
      forward_reads: illumina_forward_reads

      busco_data: busco_data
      identifier: identifier
      gtdbtk_data: gtdbtk_data
      semibin_environment: semibin_environment
      assembly:
        source:
        - workflow_pilon/pilon_polished_assembly
        - medaka/polished_assembly
        - flye/assembly
        - spades/scaffolds
        pickValue: first_non_null
      bam_file: sam_to_sorted_bam/sortedbam
      threads: threads
      memory: memory
      step:
        default: 1
    out: [bins,das_tool_output, maxbin2_output, semibin_output, metabat2_output,checkm_output, gtdbtk_output, busco_output, bins_summary_table, bins_read_stats, eukrep_fasta, eukrep_stats_file]
#############################################
#### GEM workflow
  workflow_GEM:
    label: GEM workflow
    doc: CarveMe community genomescale metabolic models workflow from bins
    when: $(inputs.binning && inputs.run_GEM)
    run: workflow_metagenomics_GEM.cwl
    in:
      binning: binning
      run_smetana: run_smetana
      run_GEM: run_GEM

      identifier: identifier
      bins: workflow_binning/bins
      threads: threads
    out: [carveme_gems_folder,protein_fasta_folder,memote_folder,smetana_output,gemstats_out]

#############################################

# OUTPUT FOLDER PREPARATION #

#############################################
#### Filtered reads output folder (keep)
  keep_readfilter_files_to_folder:
    doc: Preparation of read filtering output files to a specific output folder
    label: Read filtering output folder
    when: $(inputs.keep_filtered_reads)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      keep_filtered_reads: keep_filtered_reads

      files:
        source: [workflow_quality_nanopore/filtered_reads, workflow_quality_pacbio/filtered_reads, workflow_quality_illumina/QC_forward_reads, workflow_quality_illumina/QC_reverse_reads, workflow_quality_nanopore/filtlong_log, workflow_quality_nanopore/reference_filter_longreads_log, workflow_quality_pacbio/filtlong_log, workflow_quality_pacbio/reference_filter_longreads_log]
        linkMerge: merge_flattened
        pickValue: all_non_null
      folders:
        source: [workflow_quality_pacbio/kraken2_folder, workflow_quality_nanopore/kraken2_folder, workflow_quality_illumina/kraken2_folder, workflow_quality_illumina/reports_folder, workflow_quality_nanopore/nanoplot_unfiltered_folder, workflow_quality_nanopore/nanoplot_filtered_folder, workflow_quality_pacbio/nanoplot_unfiltered_folder, workflow_quality_pacbio/nanoplot_filtered_folder]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("Read_filtering_and_classification")
    out:
      [results]
#############################################
#### Filtered reads output folder
  readfilter_files_to_folder:
    doc: Preparation of read filtering output files to a specific output folder
    label: Read filtering output folder
    when: $(inputs.keep_filtered_reads == false)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      keep_filtered_reads: keep_filtered_reads

      files:
        source: [workflow_quality_nanopore/filtlong_log, workflow_quality_nanopore/reference_filter_longreads_log, workflow_quality_pacbio/filtlong_log, workflow_quality_pacbio/reference_filter_longreads_log]
        linkMerge: merge_flattened
        pickValue: all_non_null
      folders:
        source: [workflow_quality_pacbio/kraken2_folder, workflow_quality_nanopore/kraken2_folder, workflow_quality_illumina/kraken2_folder, workflow_quality_illumina/reports_folder, workflow_quality_nanopore/nanoplot_unfiltered_folder, workflow_quality_nanopore/nanoplot_filtered_folder, workflow_quality_pacbio/nanoplot_unfiltered_folder, workflow_quality_pacbio/nanoplot_filtered_folder]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("Read_filtering_and_classification")
    out:
      [results]

#############################################
#### SPAdes Output folder
  spades_files_to_folder:
    doc: Preparation of SPAdes output files to a specific output folder
    label: SPADES output to folder
    when: $(inputs.run_spades)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      run_spades: run_spades

      files:
        source: [compress_spades/outfile, spades/params, spades/log, spades/internal_config, spades/internal_dataset]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("SPAdes_Assembly")
    out:
      [results]
#############################################
#### Flye output folder
  flye_files_to_folder:
    doc: Preparation of Flye output files to a specific output folder
    label: Flye output folder
    when: $(inputs.run_flye)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      run_flye: run_flye

      files:
        source: [flye/assembly, flye/assembly_info, flye/flye_log, flye/params]
        linkMerge: merge_flattened
        pickValue: all_non_null
      # folders:
        # source: [workflow_flye/00_assembly, workflow_flye/10_consensus, workflow_flye/20_repeat, workflow_flye/30_contigger, workflow_flye/40_polishing]
        # linkMerge: merge_flattened
      destination:
        valueFrom: $("Flye_Assembly")
    out:
      [results]
#############################################
#### Medaka metaQUAST output folder
  metaquast_medaka_files_to_folder:
    doc: Preparation of metaQUAST output files to a specific output folder
    label: Medaka metaQUAST output folder
    when: $(inputs.run_medaka && inputs.nanopore_reads !== null && inputs.nanopore_reads.length !== 0)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      run_medaka: run_medaka
      nanopore_reads: nanopore_reads

      files:
        source: [metaquast_medaka/metaquast_report, metaquast_medaka/quastReport]
        linkMerge: merge_flattened
        pickValue: all_non_null
      folders:
        source: [metaquast_medaka/metaquast_krona, metaquast_medaka/not_aligned, metaquast_medaka/runs_per_reference]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("QUAST_Medaka_assembly_quality")
    out:
      [results]
#############################################
#### Medaka + metaquast output folder
  medaka_files_to_folder:
    doc: Preparation of Medaka output files to a specific output folder
    label: Medaka output folder
    when: $(inputs.run_medaka && inputs.nanopore_reads !== null && inputs.nanopore_reads.length !== 0)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      run_medaka: run_medaka
      nanopore_reads: nanopore_reads

      files:
        source: [medaka/polished_assembly, medaka/gaps_in_draft_coords] # , workflow_medaka/probs, workflow_medaka/calls_to_draft
        linkMerge: merge_flattened
        pickValue: all_non_null
      folders:
        source: [metaquast_medaka_files_to_folder/results]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("Medaka_assembly_polishing")
    out:
      [results]
#############################################
#### Pilon metaQUAST output folder
  metaquast_pilon_files_to_folder:
    doc: Preparation of QUAST output files to a specific output folder
    label: Illumina metaQUAST output folder
    when: $(inputs.run_pilon && inputs.illumina_forward_reads !== null && inputs.illumina_forward_reads.length !== 0)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      run_pilon: run_pilon
      illumina_forward_reads: illumina_forward_reads

      files:
        source: [metaquast_pilon/metaquast_report, metaquast_pilon/quastReport]
        linkMerge: merge_flattened
        pickValue: all_non_null
      folders:
        source: [metaquast_pilon/metaquast_krona, metaquast_pilon/not_aligned]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("QUAST_Illumina_polished_assembly_quality")
    out:
      [results]
#############################################
#### Pilon + metaQUAST output folder
  pilon_files_to_folder:
    doc: Preparation of pilon output files to a specific output folder
    label: Pilon output folder
    when: $(inputs.run_pilon && inputs.illumina_forward_reads !== null && inputs.illumina_forward_reads.length !== 0)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      run_pilon: run_pilon
      illumina_forward_reads: illumina_forward_reads

      files:
        source: [workflow_pilon/vcf, workflow_pilon/pilon_polished_assembly, workflow_pilon/log]
        linkMerge: merge_flattened
        pickValue: all_non_null
      folders:
        source: [metaquast_pilon_files_to_folder/results]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("Illumina_polished_assembly")
    out:
      [results]


#############################################
#### Combined assembly steps output folder
  assembly_files_to_folder:
    doc: Preparation of Flye output files to a specific output folder
    label: Flye output folder
    run: ../tools/expressions/files_to_folder.cwl
    in:
      folders:
        source: [spades_files_to_folder/results, flye_files_to_folder/results, medaka_files_to_folder/results, pilon_files_to_folder/results]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("Assembly")
    out:
      [results]
#############################################
#### BAM files output folder
  # sorted_bam_files_to_folder:
  #   doc: Preparation of mapped reads (sorted bam files) to a specific output folder
  #   label: Mapped reads output to folder
  #   when: $(inputs.binning)
  #   run: ../tools/expressions/files_to_folder.cwl
  #   in:
  #     binning: binning
  #     files:
  #       source: [sam_to_sorted_bam/sortedbam, bbmap/stats, contig_read_counts/contigReadCounts, bbmap/covstats, bbmap/log]
  #       linkMerge: merge_flattened
  #     destination:
  #       valueFrom: $("5_BBMAP_ReadMapping")
  #   out:
  #     [results]
#############################################
#### Binning output folder
  binning_files_to_folder:
    doc: Preparation of binning output files and folders to a specific output folder
    label: Binning output to folder
    when: $(inputs.binning)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      binning: binning

      folders:
        source: [workflow_binning/das_tool_output, workflow_binning/metabat2_output, workflow_binning/maxbin2_output, workflow_binning/semibin_output , workflow_binning/checkm_output, workflow_binning/gtdbtk_output, workflow_binning/busco_output]
        linkMerge: merge_flattened
        pickValue: all_non_null
      files:
        source: [contig_read_counts/contigReadCounts, workflow_binning/bins_read_stats, workflow_binning/bins_summary_table, workflow_binning/eukrep_fasta, workflow_binning/eukrep_stats_file]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("Binning")
    out:
      [results]
#############################################
#### GEMs output folder
  GEM_files_to_folder:
    doc: Preparation of GEM workflow output files and folders to a specific output folder
    label: GEM workflow output to folder
    when: $(inputs.binning && inputs.run_GEM)
    run: ../tools/expressions/files_to_folder.cwl
    in:
      binning: binning

      run_GEM: run_GEM

      folders:
        source: [workflow_GEM/carveme_gems_folder, workflow_GEM/protein_fasta_folder, workflow_GEM/memote_folder]
        linkMerge: merge_flattened
        pickValue: all_non_null
      files:
        source: [workflow_GEM/smetana_output, workflow_GEM/gemstats_out]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination:
        valueFrom: $("metaGEM")
    out:
      [results]
#############################################

s:author:
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-8172-8981
    s:email: mailto:jasper.koehorst@wur.nl
    s:name: Jasper Koehorst
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-9524-5964
    s:email: mailto:bart.nijsse@wur.nl
    s:name: Bart Nijsse

s:citation: https://m-unlock.nl
s:codeRepository: https://gitlab.com/m-unlock/cwl
s:dateCreated: "2020-00-00"
s:dateModified: "2023-02-10"
s:license: https://spdx.org/licenses/Apache-2.0
s:copyrightHolder: "UNLOCK - Unlocking Microbial Potential"

$namespaces:
  s: https://schema.org/